import React, { Component, PropTypes } from 'react'
import styled, { css, keyframes } from 'styled-components'
import Box from './Box'
import Pane from './Pane'
import Overlay from './Overlay'
import Close from './Close'

const rotate360InAnimation = keyframes`
  from {
    transform: translateX(100%) rotate(0deg);
  }

  to {
    transform: translateX(-100%) rotate(-360deg);
  }
`

const rotate360OutAnimation = keyframes`
  from {
    transform: translateX(-100%) rotate(0deg);
  }

  to {
    transform: translateX(100%) rotate(360deg);
  }
`

const SheetCloseStyles = ({ isClosing, theme }) => `
  cursor: pointer;
  transform: translateX(-100%);
  background-color: rgba(255, 255, 255, 0.4);
  transition: background-color ${theme.animationDuration.fast};
  animation: ${isClosing ? rotate360OutAnimation : rotate360InAnimation} ${theme.animationDuration.slow} ${theme.animationEasing.deceleration};

  &:hover {
    background-color: rgba(255, 255, 255, 0.6);
  }

  &:active {
    background-color: rgba(255, 255, 255, 0.4);
  }
`

const SheetClose = styled(({ isClosing, ...props }) =>
  <Box position='absolute' marginLeft={-4} marginTop={4} padding={2} circle {...props}>
    <Close fill='#fff' />
  </Box>
)`${SheetCloseStyles}`

const slideInAnimation = keyframes`
  from {
    transform: translateX(100%);
  }
  to {
    transform: translateX(0);
  }
`
const slideOutAnimation = keyframes`
  from {
    transform: translateX(0);
  }
  to {
    transform: translateX(100%);
  }
`

const styles = ({ slideOut, theme }) => css`
  transform: translateZ(0);
  animation: ${slideOut ? slideOutAnimation : slideInAnimation } ${theme.animationDuration.slow} ${theme.animationEasing.deceleration};
`

const SheetPane = styled(({ width, slideOut, ...props }) =>
  <Pane
    white
    position='absolute'
    shadow
    width={width}
    top={0}
    right={0}
    bottom={0}
    {...props} />)`${styles}`

SheetPane.propTypes = {
  width: PropTypes.string,
  slideOut: PropTypes.bool
}

SheetPane.displayName = 'SheetPane'

class Sheet extends Component {
  constructor() {
    super()

    this.handleClose = this.handleClose.bind(this)
    this.handleAnimationEnd = this.handleAnimationEnd.bind(this)

    this.state = {
      isClosing: false
    }
  }

  handleClose() {
    this.setState({ isClosing: true })
  }

  handleAnimationEnd() {
    if (!this.state.isClosing) return
    this.props.onClose()
    this.setState({ isClosing: false })
  }

  render() {
    const { onClose, isOpen, children, ...props } = this.props
    const { isClosing } = this.state

    if (!isOpen) return null

    return (
      <Overlay onClick={this.handleClose} fadeOut={isClosing} onAnimationEnd={this.handleAnimationEnd}>
        <SheetPane slideOut={isClosing} {...props}>
          <SheetClose isClosing={isClosing} />
          {children}
        </SheetPane>
      </Overlay>
    )
  }
}

Sheet.propTypes = {
  width: PropTypes.string,
  isOpen: PropTypes.bool,
  onClose: PropTypes.func
}

Sheet.defaultProps = {
  width: '680px',
  isOpen: true
}


Sheet.displayName = 'Sheet'

export default Sheet
